/*
   Script to open Microsoft Office files on the Application Server from
   PM Shell, Thunderbird, File Commander and etc.

   MS Office programs will be runned as Remote Application. Thist script should
   be placed into the same directory as freerdp.exe.

   Usage: MSOFFICE.CMD <MS Office file full name>


   Add next line to fc.ini ([Associations] section) to open MS Office documents
   from File Commander:

     *.{XLSX,XLSM,XLSB,XLTX,XLTXM,XLS,XLT,XLAM,XLA,DOCX,DOCM,DOC,RTF,PPTX,PPTM,
     PPT,PPS}: C:\Programs\FreeRDP\msoffice.cmd !f

   To open MS Office files from PM Shell open properties notebook for
   msoffice.cmd, go to Associations bookmark and add new names, like:
     *doc, *docx, *xlsx, etc.

   Digi, 2018-2019.
*/

/* ************************** Configuration *************************** */

server    = "remoteapp.server.dom"    /* Server name.      */
user      = "domain\username"         /* Domain\user name. */
password  = "MyPassword"              /* User password.    */

/* Path to the Microsoft Office programs on the application server. */
RADir = "C:\Program Files (x86)\Microsoft Office\Office14"

/* Local directories redirection.
   redir.n: "remoteName","local path" */

redir.0 = 0 /* Number of records */
redir.1 = '"Doc","D:\HOME\Documents"'

debug   = 1 /* Set to 1 to see FreeRDP console. */

minimize = 0 /* Set to 1 to hide initial main window. */

/* ******************************************************************** */



parse source os2_str type_str script


/* List of MS Office programs and files extensions */

progList.0 = 5

progList.1._extList = "XLSX XLSM XLSB XLTX XLTXM XLS XLT XLAM XLA"
progList.1._prog    = "EXCEL.EXE"

progList.2._extList = "DOCX DOCM DOC DOTX DOTM DOT XML RTF"
progList.2._prog    = "WINWORD.EXE"

progList.3._extList = "PPTX PPTM PPT POTX POTM POT THMX PPS PPSX PPSM PPAM PPA"
progList.3._prog    = "POWERPNT.EXE"

progList.4._extList = "MSG"
progList.4._prog    = "OUTLOOK.EXE"

progList.5._extList = "MDB ACCDB ACCDE ACCDT ACCDR"
progList.5._prog    = "MSACCESS.EXE"


/* Check the existence of the file */

file = strip( arg( 1 ), "b", '"' )
if stream( file, "c", "query exists" ) = "" then
do
  say "File " || file || " does not exist."
  say "Press any key to exit."
  "@pause >nul"
  exit 2
end


/* Get local path to the file. */

filePath = filespec( "drive", file ) || filespec( "path", file )
if right( filePath, 1 ) = '\' then
  filePath = left( filePath, length(filePath) - 1 )

if filePath = "" then
do
  say "Usage: " || filespec( "name", script ) || " <file full name>"
  exit 1
end


/* Detect MS Office program to open given file */

RAProg = progList.1._prog    /* First program in list is default. */
ext = ""
extPos = translate( lastpos( '.', file ) )
if extPos \= 0 then
do
  ext = translate( substr( file, extPos + 1 ) )
  do i = 1 to progList.0
    if wordpos( ext, progList.i._extList ) \= 0 then
    do
      RAProg = progList.i._prog
      leave
    end
  end
end


/* Make remote disk name. */

parse value time('L') with H':'M':'S'.'MS'0000'
call random ,,M || S || MS
disk = "open_" || d2x( random( 0, 100000 ) )


if minimize = 1 then
  call value "FREERDP", "MIN", "OS2ENVIRONMENT"

/*
   Make command to run FreeRDP.
   Redirect given file directory to remote disk "open". Run MS Office program
   as Remote Application.
*/

if debug = 1 then
do
  say "Run remote application (for ." || ext || "): " || RAProg
  say
  cmd = ''
end
else
  cmd = 'start /min /c '

cmd = cmd || 'ofreerdp.exe /v:"' || server || ,
      '" /u:"' || user || '" /p:"' || password || '" /gdi:hw ' || ,
      '/compression-level:2 +auto-reconnect /relax-order-checks ' || ,
      '/cert:ignore ' || ,
      '/app:program:"' || RAProg || '",workdir:"' || RADir || '",' || ,
      'cmd:''"\\tsclient\' || disk || '\' || filespec( "name", file ) || ,
      '"'' ' || '/drive:"' || disk || '","' || filePath || '"'

/* Redirect user directories from the configured list redir. */
if redir.0 \= 0 then
do i = 1 to redir.0
  cmd = cmd || " /drive:" || redir.i
end

/* Run FreeRDP */

scriptDir = filespec( "path", script )
if ( length( scriptDir ) > 1 ) & ( right( scriptDir, 1 ) = '\' ) then
  scriptDir = substr( scriptDir, 1, length(scriptDir) - 1 )

savePath = directory()
call directory filespec( "drive", script ) || scriptDir

cmd

call directory saveDir

if debug = 1 then
do
  say "Press any key to exit. Set debug=0 in " || ,
      filespec( "name", script ) || " to avoid this pause."
  '@pause > nul'
end

exit 0
